@@ -6,10 +6,9 @@ import os
 import sys
 import platform
 import tempfile
-import subprocess
 from shutil import rmtree
 
-version = '21w01a'
+version = '21w04a'
 
 def file_type(file: str) -> str:
     if(not os.path.isfile(file)):
@@ -95,6 +94,9 @@ def main_options():
     ops += add_argument('--constant_rate_factor', '-crf', type=int, default=15,
         parent='exportMediaOps', range='0 to 51',
         help='set the quality for video using the crf method.')
+    ops += add_argument('--render', default='auto',
+        choices=['av', 'opencv', 'auto'],
+        help='choice which rendering method to use. (experimental)')
 
     ops += add_argument('motionOps', nargs=0, action='grouping')
     ops += add_argument('--dilates', '-d', type=int, default=2, range='0 to 5',
@@ -118,6 +120,8 @@ def main_options():
     ops += add_argument('--export_as_json', action='store_true',
         help='export as a JSON file that can be read by auto-editor later. (experimental)')
 
+    #ops += add_argument('--zoom')
+
     ops += add_argument('--ignore', nargs='*',
         help='the range that will be marked as "loud"')
     ops += add_argument('--cut_out', nargs='*',
@@ -259,9 +263,9 @@ def main():
             sys.exit()
 
         from generateTestMedia import generateTestMedia
-        from usefulFunctions import getBinaries
+        from usefulFunctions import FFmpeg
 
-        ffmpeg, __ = getBinaries(platform.system(), dirPath, args.my_ffmpeg)
+        ffmpeg = FFmpeg(platform.system(), dirPath, args.my_ffmpeg, False)
         generateTestMedia(ffmpeg, args.output_file, args.fps, args.duration,
             args.width, args.height)
         sys.exit()
@@ -276,16 +280,18 @@ def main():
         log = Log(False, False, False)
 
         from info import getInfo
-        from usefulFunctions import getBinaries
-        ffmpeg, ffprobe = getBinaries(platform.system(), dirPath, args.my_ffmpeg)
+        from usefulFunctions import FFmpeg, FFprobe
+
+        ffmpeg = FFmpeg(platform.system(), dirPath, args.my_ffmpeg, False)
+        ffprobe = FFprobe(platform.system(), dirPath, args.my_ffmpeg)
+
         getInfo(args.input, ffmpeg, ffprobe, log)
         sys.exit()
     else:
         option_data = main_options()
         args = ParseOptions(sys.argv[1:], Log(True), option_data)
 
-    log = Log(args.debug, args.show_ffmpeg_debug, args.quiet)
-    log.debug('')
+    timer = Timer(args.quiet)
 
     # Print the help screen for the entire program.
     if(args.help):
@@ -298,16 +304,10 @@ def main():
 
     del option_data
 
-    if(args.version):
-        print('Auto-Editor version', version)
-        sys.exit()
+    from usefulFunctions import FFmpeg, FFprobe
+    ffmpeg = FFmpeg(platform.system(), dirPath, args.my_ffmpeg, args.show_ffmpeg_debug)
+    ffprobe = FFprobe(platform.system(), dirPath, args.my_ffmpeg)
 
-    from usefulFunctions import getBinaries, pipeToConsole, ffAddDebug
-    from mediaMetadata import vidTracks, getSampleRate, getAudioBitrate
-    from mediaMetadata import getVideoCodec, ffmpegFPS
-    from wavfile import read
-
-    ffmpeg, ffprobe = getBinaries(platform.system(), dirPath, args.my_ffmpeg)
     makingDataFile = (args.export_to_premiere or args.export_to_resolve or
         args.export_to_final_cut_pro or args.export_as_json)
     is64bit = '64-bit' if sys.maxsize > 2**32 else '32-bit'
@@ -315,11 +315,10 @@ def main():
     if(args.debug and args.input == []):
         print('Python Version:', platform.python_version(), is64bit)
         print('Platform:', platform.system(), platform.release())
-        # Platform can be 'Linux', 'Darwin' (macOS), 'Java', 'Windows'
-        ffmpegVersion = pipeToConsole([ffmpeg, '-version']).split('\n')[0]
+        print('FFmpeg path:', ffmpeg.getPath())
+        ffmpegVersion = ffmpeg.pipe(['-version']).split('\n')[0]
         ffmpegVersion = ffmpegVersion.replace('ffmpeg version', '').strip()
         ffmpegVersion = ffmpegVersion.split(' ')[0]
-        print('FFmpeg path:', ffmpeg)
         print('FFmpeg version:', ffmpegVersion)
         print('Auto-Editor version', version)
         sys.exit()
@@ -328,7 +327,20 @@ def main():
         log.warning('You have the 32-bit version of Python, which may lead to' \
             'memory crashes.')
 
+    if(args.version):
+        print('Auto-Editor version', version)
+        sys.exit()
+
+    TEMP = tempfile.mkdtemp()
+    log = Log(args.debug, args.show_ffmpeg_debug, args.quiet, temp=TEMP)
+    log.debug(f'\n   - Temp Directory: {TEMP}')
+
+    from mediaMetadata import vidTracks, getSampleRate, getAudioBitrate
+    from mediaMetadata import getVideoCodec, ffmpegFPS
+    from wavfile import read
+
     from usefulFunctions import isLatestVersion
+
     if(not args.quiet and isLatestVersion(version, log)):
         log.print('\nAuto-Editor is out of date. Run:\n')
         log.print('    pip3 install -U auto-editor')
@@ -341,10 +353,7 @@ def main():
     from validateInput import validInput
     inputList = validInput(args.input, ffmpeg, log)
 
-    timer = Timer(args.quiet)
-
     # Figure out the output file names.
-
     def newOutputName(oldFile: str, exa=False, data=False, exc=False) -> str:
         dotIndex = oldFile.rfind('.')
         if(exc):
@@ -361,19 +370,16 @@ def main():
             args.output_file.append(newOutputName(inputList[i],
                 args.export_as_audio, makingDataFile, args.export_as_json))
 
-    TEMP = tempfile.mkdtemp()
-    log.debug(f'\n   - Temp Directory: {TEMP}')
-
     if(args.combine_files):
         # Combine video files, then set input to 'combined.mp4'.
-        cmd = [ffmpeg, '-y']
+        cmd = []
         for fileref in inputList:
             cmd.extend(['-i', fileref])
         cmd.extend(['-filter_complex', f'[0:v]concat=n={len(inputList)}:v=1:a=1',
             '-codec:v', 'h264', '-pix_fmt', 'yuv420p', '-strict', '-2',
             f'{TEMP}/combined.mp4'])
-        cmd = ffAddDebug(cmd, log.is_ffmpeg)
-        subprocess.call(cmd)
+        ffmpeg.run(cmd)
+        del cmd
         inputList = [f'{TEMP}/combined.mp4']
 
     speeds = [args.silent_speed, args.video_speed]
@@ -423,22 +429,22 @@ def main():
                 tracks = 1
             else:
                 tracks = args.force_tracks_to
-            cmd = [ffmpeg, '-y', '-i', INPUT_FILE]
+            cmd = ['-i', INPUT_FILE]
             if(audioBitrate is not None):
                 cmd.extend(['-b:a', audioBitrate])
             cmd.extend(['-ac', '2', '-ar', sampleRate, '-vn', f'{TEMP}/fastAud.wav'])
-            cmd = ffAddDebug(cmd, log.is_ffmpeg)
-            subprocess.call(cmd)
+            ffmpeg.run(cmd)
+            del cmd
 
             sampleRate, audioData = read(f'{TEMP}/fastAud.wav')
         else:
             if(args.force_fps_to is not None):
                 fps = args.force_fps_to
-            elif(args.export_to_premiere or args.export_to_final_cut_pro):
-                # This is the default fps value for Premiere Pro Projects.
-                fps = 29.97
+            elif(args.export_to_premiere or args.export_to_final_cut_pro or
+                args.export_to_resolve):
+                # Based on timebase.
+                fps = int(ffmpegFPS(ffmpeg, INPUT_FILE, log))
             else:
-                # Don't know the correct fps to set so get the input's
                 fps = ffmpegFPS(ffmpeg, INPUT_FILE, log)
             log.debug(f'ffmpeg fps: {fps}')
 
@@ -458,28 +464,25 @@ def main():
                 log.error("You choose a track that doesn't exist.\n" \
                     f'There {message}.\n {allTracks}')
 
-            vcodec = getVideoCodec(INPUT_FILE, ffmpeg, log, args.video_codec)
-
             # Split audio tracks into: 0.wav, 1.wav, etc.
             for trackNum in range(tracks):
-                cmd = [ffmpeg, '-y', '-i', INPUT_FILE]
+                cmd = ['-i', INPUT_FILE]
                 if(audioBitrate is not None):
                     cmd.extend(['-ab', audioBitrate])
                 cmd.extend(['-ac', '2', '-ar', sampleRate, '-map',
                     f'0:a:{trackNum}', f'{TEMP}/{trackNum}.wav'])
-                cmd = ffAddDebug(cmd, log.is_ffmpeg)
-                subprocess.call(cmd)
+                ffmpeg.run(cmd)
+                del cmd
 
             # Check if the `--cut_by_all_tracks` flag has been set or not.
             if(args.cut_by_all_tracks):
                 # Combine all audio tracks into one audio file, then read.
-                cmd = [ffmpeg, '-y', '-i', INPUT_FILE, '-filter_complex',
+                cmd = ['-i', INPUT_FILE, '-filter_complex',
                     f'[0:a]amix=inputs={tracks}:duration=longest', '-ar',
                     sampleRate, '-ac', '2', '-f', 'wav', f'{TEMP}/combined.wav']
-                cmd = ffAddDebug(cmd, log.is_ffmpeg)
-                subprocess.call(cmd)
-
+                ffmpeg.run(cmd)
                 sampleRate, audioData = read(f'{TEMP}/combined.wav')
+                del cmd
             else:
                 # Read only one audio file.
                 if(os.path.isfile(f'{TEMP}/{args.cut_by_this_track}.wav')):
@@ -520,14 +523,10 @@ def main():
             hasLoud = combineArrs(audioList, motionList, args.edit_based_on, log)
             del audioList, motionList
 
-            chunks, includeFrame = applySpacingRules(hasLoud, fps,
-                args.frame_margin, args.min_clip_length, args.min_cut_length,
-                args.ignore, args.cut_out, log)
+            chunks = applySpacingRules(hasLoud, fps, args.frame_margin,
+                args.min_clip_length, args.min_cut_length, args.ignore,
+                args.cut_out, log)
             del hasLoud
-        else:
-            from cutting import generateIncludes
-
-            includeFrame = generateIncludes(chunks, log)
 
         clips = []
         numCuts = len(chunks)
@@ -542,10 +541,7 @@ def main():
                 constantLoc = INPUT_FILE[:dotIndex] + end
             else:
                 constantLoc = f'{TEMP}/constantVid{fileFormat}'
-            cmd = [ffmpeg, '-y', '-i', INPUT_FILE, '-filter:v', 'fps=fps=30',
-                constantLoc]
-            cmd = ffAddDebug(cmd, log.is_ffmpeg)
-            subprocess.call(cmd)
+            ffmpeg.run(['-i', INPUT_FILE, '-filter:v', 'fps=fps=30', constantLoc])
             INPUT_FILE = constantLoc
 
         if(args.export_as_json):
@@ -566,23 +562,39 @@ def main():
             continue
 
         if(audioFile):
-            from fastAudio import fastAudio, handleAudio
+            from fastAudio import fastAudio, handleAudio, convertAudio
             theFile = handleAudio(ffmpeg, INPUT_FILE, audioBitrate, str(sampleRate),
                 TEMP, log)
-            fastAudio(theFile, newOutput, chunks, speeds, log, fps,
+            fastAudio(theFile, f'{TEMP}/convert.wav', chunks, speeds, log, fps,
                 args.machine_readable_progress, args.no_progress)
+            convertAudio(ffmpeg, ffprobe, f'{TEMP}/convert.wav', INPUT_FILE, newOutput,
+                args, log)
             continue
 
-        from fastVideo import handleAudioTracks, fastVideo, muxVideo
-        continueVid = handleAudioTracks(ffmpeg, newOutput, args.export_as_audio,
-            tracks, args.keep_tracks_seperate, chunks, speeds, fps, TEMP,
-            args.machine_readable_progress, args.no_progress, log)
+        from videoUtils import handleAudioTracks, muxVideo
+
+        continueVid = handleAudioTracks(ffmpeg, newOutput, args, tracks, chunks, speeds,
+            fps, TEMP, log)
         if(continueVid):
-            fastVideo(INPUT_FILE, chunks, includeFrame, speeds, fps,
-            args.machine_readable_progress, args.no_progress, TEMP, log)
-            muxVideo(ffmpeg, newOutput, args.keep_tracks_seperate, tracks,
-                args.video_bitrate, args.tune, args.preset, vcodec, args.audio_codec,
-                args.constant_rate_factor, TEMP, log)
+
+            if(args.render == 'auto'):
+                try:
+                    import av
+                    args.render = 'av'
+                except ImportError:
+                    args.render = 'opencv'
+
+            log.debug(f'Using {args.render} method')
+            if(args.render == 'av'):
+                from renderVideo import renderAv
+                renderAv(ffmpeg, INPUT_FILE, args, chunks, speeds, TEMP, log)
+
+            if(args.render == 'opencv'):
+                from renderVideo import renderOpencv
+                renderOpencv(ffmpeg, INPUT_FILE, args, chunks, speeds, fps, TEMP, log)
+
+            # Now mix new audio(s) and the new video.
+            muxVideo(ffmpeg, newOutput, args, tracks, TEMP, log)
 
     if(newOutput is not None and not os.path.isfile(newOutput)):
         log.bug(f'The file {newOutput} was not created.')
