@@ -1,18 +1,29 @@
 '''fastAudio.py'''
 
-from usefulFunctions import ProgressBar, getNewLength, ffAddDebug
+from usefulFunctions import ProgressBar, getNewLength
+from mediaMetadata import getAudioCodec
+
+def convertAudio(ffmpeg, ffprobe, theFile, INPUT_FILE, outFile, args, log):
+    log.debug(f'Convering internal audio file: {theFile} to {outFile}')
+
+    realCodec = args.audio_codec
+    if(realCodec is None):
+        realCodec = getAudioCodec(ffprobe, INPUT_FILE)
+    if(realCodec == "pcm_s16le" and outFile.endswith('.m4a')):
+        log.error(f'Codec: {realCodec} is not supported in the m4a container.')
+
+    ffmpeg.run(['-i', theFile, '-acodec', realCodec, outFile])
 
 def handleAudio(ffmpeg, theFile, audioBit, samplerate: str, temp, log) -> str:
     import subprocess
 
     log.checkType(samplerate, 'samplerate', str)
-    cmd = [ffmpeg, '-y', '-i', theFile]
+    cmd = ['-i', theFile]
     if(audioBit is not None):
         cmd.extend(['-b:a', audioBit])
         log.checkType(audioBit, 'audioBit', str)
     cmd.extend(['-ac', '2', '-ar', samplerate, '-vn', f'{temp}/faAudio.wav'])
-    cmd = ffAddDebug(cmd, log.is_ffmpeg)
-    subprocess.call(cmd)
+    ffmpeg.run(cmd)
     log.conwrite('')
 
     return f'{temp}/faAudio.wav'
