@@ -1,16 +1,18 @@
 '''info.py'''
 
 from mediaMetadata import ffmpegFPS, vidTracks
-from usefulFunctions import pipeToConsole
+
+import os
 
 def getInfo(files, ffmpeg, ffprobe, log):
     for file in files:
-        print(f'file: {file}')
+        if(os.path.exists(file)):
+            print(f'file: {file}')
+        else:
+            log.error(f'Could not find file: {file}')
 
-        hasVid = pipeToConsole([ffprobe, '-v', 'error', '-show_streams',
-            '-select_streams', 'v', file])
-        hasAud = pipeToConsole([ffprobe, '-v', 'error', '-show_streams',
-            '-select_streams', 'a', file])
+        hasVid = ffprobe.pipe(['-show_streams', '-select_streams', 'v', file])
+        hasAud = ffprobe.pipe(['-show_streams', '-select_streams', 'a', file])
 
         hasAud = len(hasAud) > 5
         hasVid = len(hasVid) > 5
@@ -19,16 +21,21 @@ def getInfo(files, ffmpeg, ffprobe, log):
             fps = ffmpegFPS(ffmpeg, file, log)
             print(f' - fps: {fps}')
 
-            res = pipeToConsole([ffprobe, '-v', 'error', '-select_streams', 'v:0',
-                '-show_entries', 'stream=height,width', '-of', 'csv=s=x:p=0', file])
-            print(f' - resolution: {res.strip()}')
+            dur = ffprobe.pipe(['-i', file, '-show_entries', 'format=duration', '-of',
+                'csv=p=0']).strip()
+            print(f' - duration: {dur}')
 
-            raw_data = pipeToConsole([ffprobe, '-v', 'error', '-select_streams',
-                'v:0', '-show_entries', 'stream=codec_name,bit_rate', '-of',
+            res = ffprobe.pipe(['-select_streams', 'v:0', '-show_entries',
+                'stream=height,width', '-of', 'csv=s=x:p=0', file]).strip()
+            print(f' - resolution: {res}')
+
+            raw_data = ffprobe.pipe(['-select_streams', 'v:0', '-show_entries',
+                'stream=codec_name,bit_rate', '-of',
                 'compact=p=0:nk=1', file]).split('|')
 
             print(f' - video codec: {raw_data[0]}')
-            if(raw_data[1].isnumeric()):
+
+            if(raw_data[1].strip().isnumeric()):
                 vbit = str(int(int(raw_data[1]) / 1000)) + 'k'
             else:
                 vbit = 'N/A'
@@ -41,9 +48,9 @@ def getInfo(files, ffmpeg, ffprobe, log):
                 for track in range(tracks):
                     print(f'   - Track #{track}')
 
-                    raw_data = pipeToConsole([ffprobe, '-v', 'error', '-select_streams',
-                        f'a:{track}', '-show_entries', 'stream=codec_name,sample_rate', '-of',
-                        'compact=p=0:nk=1', file])
+                    raw_data = ffprobe.pipe(['-select_streams', f'a:{track}',
+                        '-show_entries', 'stream=codec_name,sample_rate',
+                        '-of', 'compact=p=0:nk=1', file])
 
                     raw_data = raw_data.replace('\n', '').split('|')
 
@@ -56,17 +63,17 @@ def getInfo(files, ffmpeg, ffprobe, log):
                     print(f'     - codec: {acod}')
                     print(f'     - samplerate: {sr}')
 
-                    output = pipeToConsole([ffprobe, '-v', 'error', '-select_streams',
-                        f'a:{track}', '-show_entries', 'stream=bit_rate', '-of',
-                        'compact=p=0:nk=1', file]).strip()
+                    output = ffprobe.pipe(['-select_streams', f'a:{track}',
+                        '-show_entries', 'stream=bit_rate', '-of', 'compact=p=0:nk=1',
+                        file]).strip()
                     if(output.isnumeric()):
                         abit = str(round(int(output) / 1000)) + 'k'
                         print(f'     - bitrate: {abit}')
             else:
                 print(' - audio tracks: 0')
         elif(hasAud):
-            raw_data = pipeToConsole([ffprobe, '-v', 'error', '-select_streams',
-                'a:0', '-show_entries', 'stream=codec_name,sample_rate', '-of',
+            raw_data = ffprobe.pipe(['-select_streams', 'a:0', '-show_entries',
+                'stream=codec_name,sample_rate', '-of',
                 'compact=p=0:nk=1', file]).split('|')
 
             acod = raw_data[0]
@@ -75,7 +82,7 @@ def getInfo(files, ffmpeg, ffprobe, log):
             print(f' - codec: {acod}')
             print(f' - samplerate: {sr}')
 
-            output = pipeToConsole([ffprobe, '-v', 'error', '-select_streams',
+            output = ffprobe.pipe(['-select_streams',
                 'a:0', '-show_entries', 'stream=bit_rate', '-of',
                 'compact=p=0:nk=1', file]).strip()
             if(output.isnumeric()):
